/**************************************************************************
*  Copyright (c) 2019 by Michael Fischer (www.emb4fun.de).
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  20.01.2019  mifi  First Version.
**************************************************************************/
#define __MAIN_C__

/*=======================================================================*/
/*  Includes                                                             */
/*=======================================================================*/
#include <stdio.h>

#include "sam4e_xplained_pro.h"
#include "ioport.h"
#include "smc.h"
#include "pio.h"

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

/* SRAM chip select */
#define SRAM_CHIP_SELECT      1
#define SRAM_CHIP_SELECT_2ND  3

/*
 * Set peripheral mode for one single IOPORT pin.
 * It will configure port mode and disable pin mode (but enable peripheral).
 *
 * pin IOPORT pin to configure
 * mode Mode masks to configure for the specified pin (\ref ioport_modes)
 */
#define ioport_set_pin_peripheral_mode(pin, mode) \
   do {\
      ioport_set_pin_mode(pin, mode);\
      ioport_disable_pin(pin);\
   } while (0)

/*=======================================================================*/
/*  Definition of all global Data                                        */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

/*************************************************************************/
/*  configure_sram                                                       */
/*                                                                       */
/*  Configure the SMC for SRAM access.                                   */
/*                                                                       */
/*  In    : cs  Chip select                                              */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void configure_sram (uint32_t cs)
{
   smc_set_setup_timing(SMC, cs, SMC_SETUP_NWE_SETUP(1)
                               | SMC_SETUP_NCS_WR_SETUP(1)
                               | SMC_SETUP_NRD_SETUP(1)
                               | SMC_SETUP_NCS_RD_SETUP(1));
         
   smc_set_pulse_timing(SMC, cs, SMC_PULSE_NWE_PULSE(6)
                               | SMC_PULSE_NCS_WR_PULSE(6)
                               | SMC_PULSE_NRD_PULSE(6)
                               | SMC_PULSE_NCS_RD_PULSE(6));
         
   smc_set_cycle_timing(SMC, cs, SMC_CYCLE_NWE_CYCLE(7)
                               | SMC_CYCLE_NRD_CYCLE(7));
          
   smc_set_mode(SMC, cs, SMC_MODE_READ_MODE | SMC_MODE_WRITE_MODE);
} /* configure_sram */

/*************************************************************************/
/*  init_sram_gpio                                                       */
/*                                                                       */
/*  Initialize SRAM gpio pins                                            */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void init_sram_gpio (void)
{
   ioport_set_pin_peripheral_mode(PIN_EBI_NCS1, PIN_EBI_NCS1_FLAGS);
   ioport_set_pin_peripheral_mode(PIN_EBI_NCS3, PIN_EBI_NCS3_FLAGS);
   ioport_set_pin_peripheral_mode(PIN_EBI_NRD, PIN_EBI_NRD_FLAGS);
   ioport_set_pin_peripheral_mode(PIN_EBI_NWE, PIN_EBI_NWE_FLAGS);

   ioport_set_pin_peripheral_mode(PIN_EBI_DATA_BUS_D0,  PIN_EBI_DATA_BUS_FLAGS);
   ioport_set_pin_peripheral_mode(PIN_EBI_DATA_BUS_D1,  PIN_EBI_DATA_BUS_FLAGS);
   ioport_set_pin_peripheral_mode(PIN_EBI_DATA_BUS_D2,  PIN_EBI_DATA_BUS_FLAGS);
   ioport_set_pin_peripheral_mode(PIN_EBI_DATA_BUS_D3,  PIN_EBI_DATA_BUS_FLAGS);
   ioport_set_pin_peripheral_mode(PIN_EBI_DATA_BUS_D4,  PIN_EBI_DATA_BUS_FLAGS);
   ioport_set_pin_peripheral_mode(PIN_EBI_DATA_BUS_D5,  PIN_EBI_DATA_BUS_FLAGS);
   ioport_set_pin_peripheral_mode(PIN_EBI_DATA_BUS_D6,  PIN_EBI_DATA_BUS_FLAGS);
   ioport_set_pin_peripheral_mode(PIN_EBI_DATA_BUS_D7,  PIN_EBI_DATA_BUS_FLAGS);

   ioport_set_pin_peripheral_mode(PIN_EBI_ADDR_BUS_A0,  PIN_EBI_ADDR_BUS_FLAG1);
   ioport_set_pin_peripheral_mode(PIN_EBI_ADDR_BUS_A1,  PIN_EBI_ADDR_BUS_FLAG1);
   ioport_set_pin_peripheral_mode(PIN_EBI_ADDR_BUS_A2,  PIN_EBI_ADDR_BUS_FLAG1);
   ioport_set_pin_peripheral_mode(PIN_EBI_ADDR_BUS_A3,  PIN_EBI_ADDR_BUS_FLAG1);
   ioport_set_pin_peripheral_mode(PIN_EBI_ADDR_BUS_A4,  PIN_EBI_ADDR_BUS_FLAG1);
   ioport_set_pin_peripheral_mode(PIN_EBI_ADDR_BUS_A5,  PIN_EBI_ADDR_BUS_FLAG1);
   ioport_set_pin_peripheral_mode(PIN_EBI_ADDR_BUS_A6,  PIN_EBI_ADDR_BUS_FLAG1);
   ioport_set_pin_peripheral_mode(PIN_EBI_ADDR_BUS_A7,  PIN_EBI_ADDR_BUS_FLAG1);
   ioport_set_pin_peripheral_mode(PIN_EBI_ADDR_BUS_A8,  PIN_EBI_ADDR_BUS_FLAG1);
   ioport_set_pin_peripheral_mode(PIN_EBI_ADDR_BUS_A9,  PIN_EBI_ADDR_BUS_FLAG1);
   ioport_set_pin_peripheral_mode(PIN_EBI_ADDR_BUS_A10, PIN_EBI_ADDR_BUS_FLAG1);
   ioport_set_pin_peripheral_mode(PIN_EBI_ADDR_BUS_A11, PIN_EBI_ADDR_BUS_FLAG1);
   ioport_set_pin_peripheral_mode(PIN_EBI_ADDR_BUS_A12, PIN_EBI_ADDR_BUS_FLAG1);
   ioport_set_pin_peripheral_mode(PIN_EBI_ADDR_BUS_A13, PIN_EBI_ADDR_BUS_FLAG1);
   ioport_set_pin_peripheral_mode(PIN_EBI_ADDR_BUS_A14, PIN_EBI_ADDR_BUS_FLAG2);
   ioport_set_pin_peripheral_mode(PIN_EBI_ADDR_BUS_A15, PIN_EBI_ADDR_BUS_FLAG2);
   ioport_set_pin_peripheral_mode(PIN_EBI_ADDR_BUS_A16, PIN_EBI_ADDR_BUS_FLAG2);
   ioport_set_pin_peripheral_mode(PIN_EBI_ADDR_BUS_A17, PIN_EBI_ADDR_BUS_FLAG2);
   ioport_set_pin_peripheral_mode(PIN_EBI_ADDR_BUS_A18, PIN_EBI_ADDR_BUS_FLAG2);

} /* init_sram_gpio */

/*************************************************************************/
/*  sram_init                                                            */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void sram_init (void)
{
   /* Initialize SRAM gpio pins */
   init_sram_gpio();

   /* Enable PMC clock for SMC */
   PMC->PMC_PCER0 = (1 << ID_SMC);
   
   /* SMC configuration between SRAM and SMC waveforms. */
   configure_sram(SRAM_CHIP_SELECT);
   configure_sram(SRAM_CHIP_SELECT_2ND);

} /* sram_init */

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  main                                                                 */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: never                                                        */
/*************************************************************************/
int main (void)
{
   sram_init();

   __asm("bkpt 0x70");
   
   return(0);
} /* main */

/*** EOF ***/
